Linux Power Management development
 help / color / mirror / Atom feed
From: Samuel Zhang <guoqing.zhang@amd.com>
To: <alexander.deucher@amd.com>, <christian.koenig@amd.com>,
	<rafael@kernel.org>, <len.brown@intel.com>, <pavel@kernel.org>,
	<gregkh@linuxfoundation.org>, <dakr@kernel.org>,
	<airlied@gmail.com>, <simona@ffwll.ch>, <ray.huang@amd.com>,
	<matthew.auld@intel.com>, <matthew.brost@intel.com>,
	<maarten.lankhorst@linux.intel.com>, <mripard@kernel.org>,
	<tzimmermann@suse.de>
Cc: <mario.limonciello@amd.com>, <lijo.lazar@amd.com>,
	<victor.zhao@amd.com>, <haijun.chang@amd.com>, <Qing.Ma@amd.com>,
	<Owen.Zhang2@amd.com>, <linux-pm@vger.kernel.org>,
	<linux-kernel@vger.kernel.org>, <amd-gfx@lists.freedesktop.org>,
	<dri-devel@lists.freedesktop.org>,
	"Samuel Zhang" <guoqing.zhang@amd.com>
Subject: [PATCH v4 4/5] PM: hibernate: add new api pm_transition_event()
Date: Wed, 9 Jul 2025 14:44:03 +0800	[thread overview]
Message-ID: <20250709064404.839975-5-guoqing.zhang@amd.com> (raw)
In-Reply-To: <20250709064404.839975-1-guoqing.zhang@amd.com>

dev_pm_ops.thaw() is called in following cases:
* normal case: after hibernation image has been created.
* error case 1: creation of a hibernation image has failed.
* error case 2: restoration from a hibernation image has failed.

For normal case, it is called mainly for resume storage devices for
saving the hibernation image. Other devices that are not involved
in the image saving do not need to resume the device. But since there's
no api to know which case thaw() is called, device drivers can't
conditionally resume device in thaw().

The new pm_transition_event() is such a api to query if thaw() is called
in normal case. The returned value in thaw() is:
* PM_EVENT_THAW: normal case, no need to resume non-storage devices.
* PM_EVENT_RECOVER: error case, need to resume devices.

Signed-off-by: Samuel Zhang <guoqing.zhang@amd.com>
---
 drivers/base/power/main.c |  5 +++
 include/linux/pm.h        | 85 +++++++++++++++++++++++----------------
 2 files changed, 56 insertions(+), 34 deletions(-)

diff --git a/drivers/base/power/main.c b/drivers/base/power/main.c
index 40e1d8d8a589..7e0982caa4d4 100644
--- a/drivers/base/power/main.c
+++ b/drivers/base/power/main.c
@@ -62,6 +62,11 @@ static LIST_HEAD(dpm_noirq_list);
 
 static DEFINE_MUTEX(dpm_list_mtx);
 static pm_message_t pm_transition;
+int pm_transition_event(void)
+{
+	return pm_transition.event;
+}
+EXPORT_SYMBOL_GPL(pm_transition_event);
 
 static int async_error;
 
diff --git a/include/linux/pm.h b/include/linux/pm.h
index 78855d794342..7e7b843ba823 100644
--- a/include/linux/pm.h
+++ b/include/linux/pm.h
@@ -471,58 +471,59 @@ const struct dev_pm_ops name = { \
 #define pm_ptr(_ptr) PTR_IF(IS_ENABLED(CONFIG_PM), (_ptr))
 #define pm_sleep_ptr(_ptr) PTR_IF(IS_ENABLED(CONFIG_PM_SLEEP), (_ptr))
 
-/*
- * PM_EVENT_ messages
+/**
+ * pm_transition_event() - Query the current pm transition event value.
+ *
+ * One example is used to query the reason why thaw() is called.
+ * It will return one of 2 values in this usage:
+ * * %PM_EVENT_THAW: normal case.
+ * * %PM_EVENT_RECOVER: error case.
+ *
+ * For other usage, it may return other values. See :ref:`PM_EVENT_ messages`
+ * for all possible values.
+ *
+ * Return: One of the %PM_EVENT_ messages
+ */
+int pm_transition_event(void);
+
+/**
+ * DOC: PM_EVENT_ messages
  *
  * The following PM_EVENT_ messages are defined for the internal use of the PM
  * core, in order to provide a mechanism allowing the high level suspend and
  * hibernation code to convey the necessary information to the device PM core
  * code:
  *
- * ON		No transition.
+ * %PM_EVENT_ON:		No transition.
  *
- * FREEZE	System is going to hibernate, call ->prepare() and ->freeze()
- *		for all devices.
+ * %PM_EVENT_FREEZE:	System is going to hibernate, call ->prepare() and
+ *		->freeze() for all devices.
  *
- * SUSPEND	System is going to suspend, call ->prepare() and ->suspend()
- *		for all devices.
+ * %PM_EVENT_SUSPEND:	System is going to suspend, call ->prepare() and
+ *		->suspend() for all devices.
  *
- * HIBERNATE	Hibernation image has been saved, call ->prepare() and
+ * %PM_EVENT_HIBERNATE:	Hibernation image has been saved, call ->prepare() and
  *		->poweroff() for all devices.
  *
- * QUIESCE	Contents of main memory are going to be restored from a (loaded)
- *		hibernation image, call ->prepare() and ->freeze() for all
+ * %PM_EVENT_QUIESCE:	Contents of main memory are going to be restored from
+ *		a (loaded) hibernation image, call ->prepare() and ->freeze() for all
  *		devices.
  *
- * RESUME	System is resuming, call ->resume() and ->complete() for all
- *		devices.
+ * %PM_EVENT_RESUME:	System is resuming, call ->resume() and ->complete()
+ *		for all devices.
  *
- * THAW		Hibernation image has been created, call ->thaw() and
+ * %PM_EVENT_THAW:		Hibernation image has been created, call ->thaw() and
  *		->complete() for all devices.
  *
- * RESTORE	Contents of main memory have been restored from a hibernation
- *		image, call ->restore() and ->complete() for all devices.
+ * %PM_EVENT_RESTORE:	Contents of main memory have been restored from a
+ *		hibernation image, call ->restore() and ->complete() for all devices.
  *
- * RECOVER	Creation of a hibernation image or restoration of the main
- *		memory contents from a hibernation image has failed, call
+ * %PM_EVENT_RECOVER:	Creation of a hibernation image or restoration of the
+ *		main memory contents from a hibernation image has failed, call
  *		->thaw() and ->complete() for all devices.
- *
- * The following PM_EVENT_ messages are defined for internal use by
- * kernel subsystems.  They are never issued by the PM core.
- *
- * USER_SUSPEND		Manual selective suspend was issued by userspace.
- *
- * USER_RESUME		Manual selective resume was issued by userspace.
- *
- * REMOTE_WAKEUP	Remote-wakeup request was received from the device.
- *
- * AUTO_SUSPEND		Automatic (device idle) runtime suspend was
- *			initiated by the subsystem.
- *
- * AUTO_RESUME		Automatic (device needed) runtime resume was
- *			requested by a driver.
  */
-
 #define PM_EVENT_INVALID	(-1)
 #define PM_EVENT_ON		0x0000
 #define PM_EVENT_FREEZE		0x0001
@@ -537,6 +538,22 @@ const struct dev_pm_ops name = { \
 #define PM_EVENT_REMOTE		0x0200
 #define PM_EVENT_AUTO		0x0400
 
+/*
+ * The following PM_EVENT_ messages are defined for internal use by
+ * kernel subsystems.  They are never issued by the PM core.
+ *
+ * USER_SUSPEND	Manual selective suspend was issued by userspace.
+ *
+ * USER_RESUME	Manual selective resume was issued by userspace.
+ *
+ * REMOTE_WAKEUP	Remote-wakeup request was received from the device.
+ *
+ * AUTO_SUSPEND	Automatic (device idle) runtime suspend was
+ *			initiated by the subsystem.
+ *
+ * AUTO_RESUME	Automatic (device needed) runtime resume was
+ *			requested by a driver.
+ */
 #define PM_EVENT_SLEEP		(PM_EVENT_SUSPEND | PM_EVENT_HIBERNATE)
 #define PM_EVENT_USER_SUSPEND	(PM_EVENT_USER | PM_EVENT_SUSPEND)
 #define PM_EVENT_USER_RESUME	(PM_EVENT_USER | PM_EVENT_RESUME)
@@ -471,58 +471,59 @@ const struct dev_pm_ops name = { \
  *
- * The following PM_EVENT_ messages are defined for the internal use of the PM
+ * The possible return values of %pm_transition_event().
+ *
+ * The following PM_EVENT_ messages are defined for the use of drivers and PM
  * core, in order to provide a mechanism allowing the high level suspend and
  * hibernation code to convey the necessary information to the device PM core
-- 
2.43.5


  parent reply	other threads:[~2025-07-09  6:44 UTC|newest]

Thread overview: 9+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2025-07-09  6:43 [PATCH v4 0/5] reduce system memory requirement for hibernation Samuel Zhang
2025-07-09  6:44 ` [PATCH v4 1/5] drm/ttm: add new api ttm_device_prepare_hibernation() Samuel Zhang
2025-07-09  9:13   ` Christian König
2025-07-09  6:44 ` [PATCH v4 2/5] drm/amdgpu: move GTT to shmem after eviction for hibernation Samuel Zhang
2025-07-09  6:44 ` [PATCH v4 3/5] PM: hibernate: shrink shmem pages after dev_pm_ops.prepare() Samuel Zhang
2025-07-09  6:44 ` Samuel Zhang [this message]
2025-07-09  7:07   ` [PATCH v4 4/5] PM: hibernate: add new api pm_transition_event() Rafael J. Wysocki
2025-07-09  6:44 ` [PATCH v4 5/5] drm/amdgpu: do not resume device in thaw for normal hibernation Samuel Zhang
2025-07-09  7:13   ` Rafael J. Wysocki

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20250709064404.839975-5-guoqing.zhang@amd.com \
    --to=guoqing.zhang@amd.com \
    --cc=Owen.Zhang2@amd.com \
    --cc=Qing.Ma@amd.com \
    --cc=airlied@gmail.com \
    --cc=alexander.deucher@amd.com \
    --cc=amd-gfx@lists.freedesktop.org \
    --cc=christian.koenig@amd.com \
    --cc=dakr@kernel.org \
    --cc=dri-devel@lists.freedesktop.org \
    --cc=gregkh@linuxfoundation.org \
    --cc=haijun.chang@amd.com \
    --cc=len.brown@intel.com \
    --cc=lijo.lazar@amd.com \
    --cc=linux-kernel@vger.kernel.org \
    --cc=linux-pm@vger.kernel.org \
    --cc=maarten.lankhorst@linux.intel.com \
    --cc=mario.limonciello@amd.com \
    --cc=matthew.auld@intel.com \
    --cc=matthew.brost@intel.com \
    --cc=mripard@kernel.org \
    --cc=pavel@kernel.org \
    --cc=rafael@kernel.org \
    --cc=ray.huang@amd.com \
    --cc=simona@ffwll.ch \
    --cc=tzimmermann@suse.de \
    --cc=victor.zhao@amd.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox